<?php

namespace Autos\Car\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Autos\Car\Http\Requests\CarRequest;
use Autos\Car\Interfaces\CarRepositoryInterface;
use Autos\Car\Models\Car;
use Autos\Car\Forms\Car as Form;

/**
 * APIController  class for car.
 */
class CarAPIController extends BaseController
{

    /**
     * Initialize car resource controller.
     *
     * @param type CarRepositoryInterface $car
     *
     * @return null
     */
    public function __construct(CarRepositoryInterface $car)
    {
        parent::__construct();
        $this->repository = $car;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Autos\Car\Repositories\Criteria\CarResourceCriteria::class);
    }

    /**
     * Display a list of car.
     *
     * @return Response
     */
    public function index(CarRequest $request)
    {
        return $this->repository
            ->setPresenter(\Autos\Car\Repositories\Presenter\CarPresenter::class)
            ->paginate();
    }

    /**
     * Display car.
     *
     * @param Request $request
     * @param Model   $car
     *
     * @return Response
     */
    public function show(CarRequest $request, Car $car)
    {
        return $car->setPresenter(\Autos\Car\Repositories\Presenter\CarListPresenter::class);
        ;
    }

    /**
     * Create new car.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CarRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('car::car.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('car/car/' . $car->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('car/car');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the car.
     *
     * @param Request $request
     * @param Model   $car
     *
     * @return Response
     */
    public function update(CarRequest $request, Car $car)
    {
        try {
            $data = $request->all();

            $car->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('car::car.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('car/car/' . $car->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('car/car/' . $car->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the car.
     *
     * @param Model   $car
     *
     * @return Response
     */
    public function destroy(CarRequest $request, Car $car)
    {
        try {
            $car->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('car::car.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('car/car/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('car/car/' . $car->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
